/*******************************************************************************
 * Copyright 2019 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs.backend;

import at.gv.egiz.moazs.client.ClientFactory;
import at.gv.egiz.moazs.repository.DeliveryRepository;
import at.gv.egiz.moazs.scheme.Msg2MzsConverter;
import at.gv.egiz.moazs.scheme.MsgResponse;
import at.gv.zustellung.app2mzs.xsd.ClientType;
import at.gv.zustellung.app2mzs.xsd.Mzs2AppPortType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;

/**
 * @author Christof Rabensteiner
 *
 */
@Component
public class ForwardResponseToServiceSink {

    private static final Logger log = LoggerFactory.getLogger(ForwardResponseToServiceSink.class);

    private final ClientFactory factory;
    private final DeliveryRepository repository;
    private final Msg2MzsConverter converter;

    @Autowired
    public ForwardResponseToServiceSink(ClientFactory factory, DeliveryRepository repository, Msg2MzsConverter converter) {
        this.factory = factory;
        this.repository = repository;
        this.converter = converter;
    }

    public CompletableFuture<Void> send(MsgResponse msgResponse, ClientType params) {
        log.info("Forward msg:{} with AppDeliveryID={} to service at {}.",
                msgResponse.getRootElementLocalPart(), msgResponse.getAppDeliveryID(), params.getURL());
        var binaryResponse = repository.retrieveBinaryResponse(msgResponse.getResponseID());
        Mzs2AppPortType client = factory.createSoap11(params, Mzs2AppPortType.class);
        return msgResponse.sendToAppClient(converter, binaryResponse, client);
    }
}
